(function() {

  /**
   * Robb
   *
   * @constructor
   */
  var Robb = function() {

    // Version
    this.version = [0, 1, 0].join('.');

  };

  [
    'function',
    'string',
    'number',
    'undefined'
  ].forEach(function (type) {

    // Name of the method, for instance “isFunction”
    var method = 'is' + type.charAt(0).toUpperCase() + type.slice(1);

    Robb.prototype[ method ] = function() {
      for ( var i = 0, len = arguments.length; i < len; i++ ) {
        if ( typeof arguments[i] !== type ) {
          return false;
        }
      }
      return true;
    };

  });

  /**
   * Robb.isArray
   */
  Robb.prototype.isArray = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( Object.prototype.toString.apply( arguments[i] ) !== '[object Array]' ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isDefined
   */
  Robb.prototype.isDefined = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( this.isUndefined( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isNull
   */
  Robb.prototype.isNull = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( arguments[i] !== null ) {
        return false;
      }
    }  
    return true;
  };

  /**
   * Robb.isDate
   */
  Robb.prototype.isDate = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( Object.prototype.toString.apply( arguments[i] ) !== '[object Date]' ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isElement
   */
  Robb.prototype.isElement = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !arguments[i] || !arguments[i].tagName || !arguments[i].nodeType ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isObject
   */
  Robb.prototype.isObject = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( this.isElement( arguments[i] ) || Object.prototype.toString.call( arguments[i] ) !== '[object Object]' ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isBoolean
   */
  Robb.prototype.isBoolean = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( arguments[i] !== true && arguments[i] !== false ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isRegExp
   */
  Robb.prototype.isRegExp = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !arguments[i] || !arguments[i].test || arguments[i].exec ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isArguments
   */
  Robb.prototype.isArguments = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !arguments[i] || !arguments[i].hasOwnProperty('callee') ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isEmpty
   */
  Robb.prototype.isEmpty = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {

      // Arrays and strings
      if ( ( this.isArray( arguments[i] ) || this.isString( arguments[i] ) ) && arguments[i].length !== 0 ) {
        return false;
      }

      // Objects
      if ( this.isObject( arguments[i] ) ) {
        for ( var key in arguments[i] ) {
          if ( arguments[i].hasOwnProperty( key ) ) {
            return false;
          }
        }
      }

    }
    return true;
  };

  /**
   * Robb.isEmail
   */
  Robb.prototype.isEmail = function() {
    var re = /^(?:[\w\!\#\$\%\&\'\*\+\-\/\=\?\^\`\{\|\}\~]+\.)*[\w\!\#\$\%\&\'\*\+\-\/\=\?\^\`\{\|\}\~]+@(?:(?:(?:[a-zA-Z0-9](?:[a-zA-Z0-9\-](?!\.)){0,61}[a-zA-Z0-9]?\.)+[a-zA-Z0-9](?:[a-zA-Z0-9\-](?!$)){0,61}[a-zA-Z0-9]?)|(?:\[(?:(?:[01]?\d{1,2}|2[0-4]\d|25[0-5])\.){3}(?:[01]?\d{1,2}|2[0-4]\d|25[0-5])\]))$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isUrl
   */
  Robb.prototype.isUrl = function() {
    var re = /^(?:(?:ht|f)tp(?:s?)\:\/\/|~\/|\/)?(?:\w+:\w+@)?((?:(?:[-\w\d{1-3}]+\.)+(?:com|org|cat|coop|int|pro|tel|xxx|net|gov|mil|biz|info|mobi|name|aero|jobs|edu|co\.uk|ac\.uk|it|fr|tv|museum|asia|local|travel|[a-z]{2})?)|((\b25[0-5]\b|\b[2][0-4][0-9]\b|\b[0-1]?[0-9]?[0-9]\b)(\.(\b25[0-5]\b|\b[2][0-4][0-9]\b|\b[0-1]?[0-9]?[0-9]\b)){3}))(?::[\d]{1,5})?(?:(?:(?:\/(?:[-\w~!$+|.,=]|%[a-f\d]{2})+)+|\/)+|\?|#)?(?:(?:\?(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)(?:&(?:[-\w~!$+|.,*:]|%[a-f\d{2}])+=?(?:[-\w~!$+|.,*:=]|%[a-f\d]{2})*)*)*(?:#(?:[-\w~!$ |\/.,*:;=]|%[a-f\d]{2})*)?$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isAlpha
   */
  Robb.prototype.isAlpha = function() {
    var re = /^[a-zA-Z]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isAlphanumeric
   */
  Robb.prototype.isAlphanumeric = function() {
    var re = /^[a-zA-Z0-9]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isEven
   */
  Robb.prototype.isEven = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isNumber( arguments[i] ) || ( arguments[i] & 1 ) !== 0 ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isOdd
   */
  Robb.prototype.isOdd = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( this.isEven( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isPrime
   */
  Robb.prototype.isPrime = function() {
    
    // References to this
    var self = this;

    // Goes through arguments
    for ( var i = 0, len = arguments.length; i < len; i++ ) {

      // If passed argument isn't a number, returns false
      if ( !self.isNumber( arguments[i] ) ) {
        return false;       
      }

      // If passed argument is a decimal number (let's say 12.45), returns false
      if ( !self.isUnsignedInt( arguments[i] ) ) {
        return false;
      }

      // If passed argument is less than 1, returns false
      if ( arguments[i] <= 1 ) {
        return false;
      }

      // If passed argument equals two, returns true
      if ( arguments[i] === 2 ) {
        return true;
      }

      // If passed argument is even, returns false
      if ( this.isEven( arguments[i] ) ) {
        return false;
      }

      // Goes through
      for ( var x = 3, max = Math.sqrt( arguments[i] ); x <= max; x += 2 ) {
        if ( ( arguments[i] % x ) === 0 ) {
          return false;
        }
      }

    }

    return true;

  };

  /**
   * Robb.isIp
   */
  Robb.prototype.isIp = function() {
    var re = /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isLowercase
   */
  Robb.prototype.isLowercase = function() {
    var re = /^[a-z0-9]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isUppercase
   */
  Robb.prototype.isUppercase = function() {
    var re = /^[A-Z0-9]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isString( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isDecimal
   */
  Robb.prototype.isDecimal = function() {
    var re = /^[0-9]+(\.[0-9]{1,2})?$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isUnsignedInt
   */
  Robb.prototype.isUnsignedInt = function() {
    var re = /^[0-9]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isInt
   */
  Robb.prototype.isInt = function() {
    var re = /^-?[0-9]+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isPercentage
   */
  Robb.prototype.isPercentage = function() {
    var re = /^-?[0-9]{0,2}(\.[0-9]{1,2})?$|^-?(100)(\.[0]{1,2})?$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isPositive
   */
  Robb.prototype.isPositive = function() {
    var re = /^\d+$/;
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( !this.isNumber( arguments[i] ) || !re.test( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isNegative
   */
  Robb.prototype.isNegative = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( this.isPositive( arguments[i] ) ) {
        return false;
      }
    }
    return true;
  };

  /**
   * Robb.isWindow
   */
  Robb.prototype.isWindow = function() {
    for ( var i = 0, len = arguments.length; i < len; i++ ) {
      if ( Object.prototype.toString.call( arguments[i] ) !== '[object global]' || !'setInterval' in arguments[i]  && !'setTimeout' in arguments[i] ) {
        return false;
      }
    }
    return true;
  };

  // Exports
  if ( typeof module !== 'undefined' && module.exports ) {
    module.exports = new Robb();
  } else {
    this.Robb = new Robb();
  }

}());